@@ -2,48 +2,48 @@
 
 """A Python package for parsing DMARC reports"""
 
-import logging
-import os
-import shutil
-import xml.parsers.expat as expat
-import json
-from datetime import datetime
-from time import sleep
-from collections import OrderedDict
-from io import BytesIO, StringIO
-from gzip import GzipFile
-from socket import timeout
-import zipfile
-from csv import DictWriter
-import re
-from base64 import b64decode
 import binascii
 import email
-import tempfile
 import email.utils
+import json
+import logging
 import mailbox
+import os
+import re
+import shutil
+import tempfile
+import xml.parsers.expat as expat
+import zipfile
+from base64 import b64decode
+from collections import OrderedDict
+from csv import DictWriter
+from datetime import datetime
+from gzip import GzipFile
+from io import BytesIO, StringIO
+from typing import Callable
 
 import mailparser
-from expiringdict import ExpiringDict
 import xmltodict
+from expiringdict import ExpiringDict
 from lxml import etree
-from mailsuite.imap import IMAPClient
 from mailsuite.smtp import send_email
-from imapclient.exceptions import IMAPClientError
 
+from parsedmarc.mail import MailboxConnection
 from parsedmarc.utils import get_base_domain, get_ip_address_info
 from parsedmarc.utils import is_outlook_msg, convert_outlook_msg
-from parsedmarc.utils import timestamp_to_human, human_timestamp_to_datetime
 from parsedmarc.utils import parse_email
+from parsedmarc.utils import timestamp_to_human, human_timestamp_to_datetime
 
-__version__ = "7.0.1"
+__version__ = "8.0.0"
 
-logging.basicConfig(
-    format='%(levelname)8s:%(filename)s:%(lineno)d:'
-           '%(message)s',
+formatter = logging.Formatter(
+    fmt='%(levelname)8s:%(filename)s:%(lineno)d:%(message)s',
     datefmt='%Y-%m-%d:%H:%M:%S')
+handler = logging.StreamHandler()
+handler.setFormatter(formatter)
 
 logger = logging.getLogger("parsedmarc")
+logger.addHandler(handler)
 logger.debug("parsedmarc v{0}".format(__version__))
 
 feedback_report_regex = re.compile(r"^([\w\-]+): (.+)$", re.MULTILINE)
@@ -73,14 +73,15 @@ class InvalidForensicReport(InvalidDMARCReport):
     """Raised when an invalid DMARC forensic report is encountered"""
 
 
-def _parse_report_record(record, offline=False, nameservers=None,
-                         dns_timeout=2.0, parallel=False):
+def _parse_report_record(record, ip_db_path=None, offline=False,
+                         nameservers=None, dns_timeout=2.0, parallel=False):
     """
     Converts a record from a DMARC aggregate report into a more consistent
     format
 
     Args:
         record (OrderedDict): The record to convert
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not query online for geolocation or DNS
         nameservers (list): A list of one or more nameservers to use
         (Cloudflare's public DNS resolvers by default)
@@ -93,6 +94,7 @@ def _parse_report_record(record, offline=False, nameservers=None,
     new_record = OrderedDict()
     new_record_source = get_ip_address_info(record["row"]["source_ip"],
                                             cache=IP_ADDRESS_CACHE,
+                                            ip_db_path=ip_db_path,
                                             offline=offline,
                                             nameservers=nameservers,
                                             timeout=dns_timeout,
@@ -203,18 +205,20 @@ def _parse_report_record(record, offline=False, nameservers=None,
     return new_record
 
 
-def parse_aggregate_report_xml(xml, offline=False, nameservers=None,
-                               timeout=2.0, parallel=False, server=None):
+def parse_aggregate_report_xml(xml, ip_db_path=None, offline=False,
+                               nameservers=None, timeout=2.0,
+                               parallel=False, keep_alive=None):
     """Parses a DMARC XML report string and returns a consistent OrderedDict
 
     Args:
         xml (str): A string of DMARC aggregate report XML
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not query online for geolocation or DNS
         nameservers (list): A list of one or more nameservers to use
         (Cloudflare's public DNS resolvers by default)
         timeout (float): Sets the DNS timeout in seconds
         parallel (bool): Parallel processing
-        server (IMAPClient): Connection object
+        keep_alive (callable): Keep alive function
 
     Returns:
         OrderedDict: The parsed aggregate DMARC report
@@ -308,12 +312,13 @@ def parse_aggregate_report_xml(xml, offline=False, nameservers=None,
 
         if type(report["record"]) == list:
             for i in range(len(report["record"])):
-                if server is not None and i > 0 and i % 20 == 0:
-                    logger.debug("Sending noop cmd")
-                    server.noop()
+                if keep_alive is not None and i > 0 and i % 20 == 0:
+                    logger.debug("Sending keepalive cmd")
+                    keep_alive()
                     logger.debug("Processed {0}/{1}".format(
                         i, len(report["record"])))
                 report_record = _parse_report_record(report["record"][i],
+                                                     ip_db_path=ip_db_path,
                                                      offline=offline,
                                                      nameservers=nameservers,
                                                      dns_timeout=timeout,
@@ -322,6 +327,7 @@ def parse_aggregate_report_xml(xml, offline=False, nameservers=None,
 
         else:
             report_record = _parse_report_record(report["record"],
+                                                 ip_db_path=ip_db_path,
                                                  offline=offline,
                                                  nameservers=nameservers,
                                                  dns_timeout=timeout,
@@ -391,21 +397,23 @@ def extract_xml(input_):
     return xml
 
 
-def parse_aggregate_report_file(_input, offline=False, nameservers=None,
+def parse_aggregate_report_file(_input, offline=False, ip_db_path=None,
+                                nameservers=None,
                                 dns_timeout=2.0,
                                 parallel=False,
-                                server=None):
+                                keep_alive=None):
     """Parses a file at the given path, a file-like object. or bytes as a
     aggregate DMARC report
 
     Args:
         _input: A path to a file, a file like object, or bytes
         offline (bool): Do not query online for geolocation or DNS
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         nameservers (list): A list of one or more nameservers to use
         (Cloudflare's public DNS resolvers by default)
         dns_timeout (float): Sets the DNS timeout in seconds
         parallel (bool): Parallel processing
-        server (IMAPClient): Connection object
+        keep_alive (callable): Keep alive function
 
     Returns:
         OrderedDict: The parsed DMARC aggregate report
@@ -413,11 +421,12 @@ def parse_aggregate_report_file(_input, offline=False, nameservers=None,
     xml = extract_xml(_input)
 
     return parse_aggregate_report_xml(xml,
+                                      ip_db_path=ip_db_path,
                                       offline=offline,
                                       nameservers=nameservers,
                                       timeout=dns_timeout,
                                       parallel=parallel,
-                                      server=server)
+                                      keep_alive=keep_alive)
 
 
 def parsed_aggregate_reports_to_csv_rows(reports):
@@ -560,7 +569,8 @@ def parsed_aggregate_reports_to_csv(reports):
 
 
 def parse_forensic_report(feedback_report, sample, msg_date,
-                          offline=False, nameservers=None, dns_timeout=2.0,
+                          offline=False, ip_db_path=None,
+                          nameservers=None, dns_timeout=2.0,
                           strip_attachment_payloads=False,
                           parallel=False):
     """
@@ -568,6 +578,7 @@ def parse_forensic_report(feedback_report, sample, msg_date,
 
     Args:
         feedback_report (str): A message's feedback report as a string
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not query online for geolocation or DNS
         sample (str): The RFC 822 headers or RFC 822 message sample
         msg_date (str): The message's date header
@@ -619,6 +630,7 @@ def parse_forensic_report(feedback_report, sample, msg_date,
 
         ip_address = re.split(r'\s', parsed_report["source_ip"]).pop(0)
         parsed_report_source = get_ip_address_info(ip_address,
+                                                   ip_db_path=ip_db_path,
                                                    offline=offline,
                                                    nameservers=nameservers,
                                                    timeout=dns_timeout,
@@ -747,21 +759,23 @@ def parsed_forensic_reports_to_csv(reports):
     return csv_file.getvalue()
 
 
-def parse_report_email(input_, offline=False, nameservers=None,
-                       dns_timeout=2.0, strip_attachment_payloads=False,
-                       parallel=False, server=None):
+def parse_report_email(input_, offline=False, ip_db_path=None,
+                       nameservers=None, dns_timeout=2.0,
+                       strip_attachment_payloads=False,
+                       parallel=False, keep_alive=None):
     """
     Parses a DMARC report from an email
 
     Args:
         input_: An emailed DMARC report in RFC 822 format, as bytes or a string
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not query online for geolocation on DNS
         nameservers (list): A list of one or more nameservers to use
         dns_timeout (float): Sets the DNS timeout in seconds
         strip_attachment_payloads (bool): Remove attachment payloads from
         forensic report results
         parallel (bool): Parallel processing
-        server (IMAPClient): Connection object
+        keep_alive (callable): keep alive function
 
     Returns:
         OrderedDict:
@@ -824,11 +838,12 @@ def parse_report_email(input_, offline=False, nameservers=None,
                     ns = nameservers
                     aggregate_report = parse_aggregate_report_file(
                         payload,
+                        ip_db_path=ip_db_path,
                         offline=offline,
                         nameservers=ns,
                         dns_timeout=dns_timeout,
                         parallel=parallel,
-                        server=server)
+                        keep_alive=keep_alive)
                     result = OrderedDict([("report_type", "aggregate"),
                                           ("report", aggregate_report)])
                     return result
@@ -877,8 +892,8 @@ def parse_report_email(input_, offline=False, nameservers=None,
 
 
 def parse_report_file(input_, nameservers=None, dns_timeout=2.0,
-                      strip_attachment_payloads=False,
-                      offline=False, parallel=False, server=None):
+                      strip_attachment_payloads=False, ip_db_path=None,
+                      offline=False, parallel=False, keep_alive=None):
     """Parses a DMARC aggregate or forensic file at the given path, a
     file-like object. or bytes
 
@@ -889,9 +904,10 @@ def parse_report_file(input_, nameservers=None, dns_timeout=2.0,
         dns_timeout (float): Sets the DNS timeout in seconds
         strip_attachment_payloads (bool): Remove attachment payloads from
         forensic report results
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not make online queries for geolocation or DNS
         parallel (bool): Parallel processing
-        server (IMAPClient): Connection object
+        keep_alive (callable): Keep alive function
 
     Returns:
         OrderedDict: The parsed DMARC report
@@ -908,23 +924,25 @@ def parse_report_file(input_, nameservers=None, dns_timeout=2.0,
     file_object.close()
     try:
         report = parse_aggregate_report_file(content,
+                                             ip_db_path=ip_db_path,
                                              offline=offline,
                                              nameservers=nameservers,
                                              dns_timeout=dns_timeout,
                                              parallel=parallel,
-                                             server=server)
+                                             keep_alive=keep_alive)
         results = OrderedDict([("report_type", "aggregate"),
                                ("report", report)])
     except InvalidAggregateReport:
         try:
             sa = strip_attachment_payloads
             results = parse_report_email(content,
+                                         ip_db_path=ip_db_path,
                                          offline=offline,
                                          nameservers=nameservers,
                                          dns_timeout=dns_timeout,
                                          strip_attachment_payloads=sa,
                                          parallel=parallel,
-                                         server=server)
+                                         keep_alive=keep_alive)
         except InvalidDMARCReport:
             raise InvalidDMARCReport("Not a valid aggregate or forensic "
                                      "report")
@@ -933,7 +951,9 @@ def parse_report_file(input_, nameservers=None, dns_timeout=2.0,
 
 def get_dmarc_reports_from_mbox(input_, nameservers=None, dns_timeout=2.0,
                                 strip_attachment_payloads=False,
-                                offline=False, parallel=False):
+                                ip_db_path=None,
+                                offline=False,
+                                parallel=False):
     """Parses a mailbox in mbox format containing e-mails with attached
     DMARC reports
 
@@ -944,6 +964,7 @@ def get_dmarc_reports_from_mbox(input_, nameservers=None, dns_timeout=2.0,
         dns_timeout (float): Sets the DNS timeout in seconds
         strip_attachment_payloads (bool): Remove attachment payloads from
         forensic report results
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not make online queries for geolocation or DNS
         parallel (bool): Parallel processing
 
@@ -968,6 +989,7 @@ def get_dmarc_reports_from_mbox(input_, nameservers=None, dns_timeout=2.0,
             try:
                 sa = strip_attachment_payloads
                 parsed_email = parse_report_email(msg_content,
+                                                  ip_db_path=ip_db_path,
                                                   offline=offline,
                                                   nameservers=nameservers,
                                                   dns_timeout=dns_timeout,
@@ -985,69 +1007,38 @@ def get_dmarc_reports_from_mbox(input_, nameservers=None, dns_timeout=2.0,
                         ("forensic_reports", forensic_reports)])
 
 
-def get_imap_capabilities(server):
+def get_dmarc_reports_from_mailbox(connection: MailboxConnection,
+                                   reports_folder="INBOX",
+                                   archive_folder="Archive",
+                                   delete=False,
+                                   test=False,
+                                   ip_db_path=None,
+                                   offline=False,
+                                   nameservers=None,
+                                   dns_timeout=6.0,
+                                   strip_attachment_payloads=False,
+                                   results=None,
+                                   batch_size=None,
+                                   create_folders=True):
     """
-    Returns a list of an IMAP server's capabilities
+    Fetches and parses DMARC reports from a mailbox
 
     Args:
-        server (imapclient.IMAPClient): An instance of imapclient.IMAPClient
-
-    Returns (list): A list of capabilities
-    """
-
-    capabilities = list(map(str, list(server.capabilities())))
-    for i in range(len(capabilities)):
-        capabilities[i] = str(capabilities[i]).replace("b'",
-                                                       "").replace("'",
-                                                                   "")
-    logger.debug("IMAP server supports: {0}".format(capabilities))
-
-    return capabilities
-
-
-def get_dmarc_reports_from_inbox(connection=None,
-                                 host=None,
-                                 user=None,
-                                 password=None,
-                                 port=None,
-                                 ssl=True,
-                                 verify=True,
-                                 timeout=30,
-                                 max_retries=4,
-                                 reports_folder="INBOX",
-                                 archive_folder="Archive",
-                                 delete=False,
-                                 test=False,
-                                 offline=False,
-                                 nameservers=None,
-                                 dns_timeout=6.0,
-                                 strip_attachment_payloads=False,
-                                 results=None,
-                                 batch_size=None):
-    """
-    Fetches and parses DMARC reports from an inbox
-
-    Args:
-        connection: An IMAPClient connection to reuse
-        host: The mail server hostname or IP address
-        user: The mail server user
-        password: The mail server password
-        port: The mail server port
-        ssl (bool): Use SSL/TLS
-        verify (bool): Verify SSL/TLS certificate
-        timeout (float): IMAP timeout in seconds
-        max_retries (int): The maximum number of retries after a timeout
-        reports_folder: The IMAP folder where reports can be found
+        connection: A Mailbox connection object
+        reports_folder: The folder where reports can be found
         archive_folder: The folder to move processed mail to
         delete (bool): Delete  messages after processing them
         test (bool): Do not move or delete messages after processing them
-        offline (bool): Do not query onfline for geolocation or DNS
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
+        offline (bool): Do not query online for geolocation or DNS
         nameservers (list): A list of DNS nameservers to query
         dns_timeout (float): Set the DNS query timeout
         strip_attachment_payloads (bool): Remove attachment payloads from
-        forensic report results
+          forensic report results
         results (dict): Results from the previous run
         batch_size (int): Number of messages to read and process before saving
+        create_folders (bool): Whether to create the destination folders
+          (not used in watch)
 
     Returns:
         OrderedDict: Lists of ``aggregate_reports`` and ``forensic_reports``
@@ -1055,9 +1046,8 @@ def get_dmarc_reports_from_inbox(connection=None,
     if delete and test:
         raise ValueError("delete and test options are mutually exclusive")
 
-    if connection is None and (user is None or password is None):
-        raise ValueError("Must supply a connection, or a username and "
-                         "password")
+    if connection is None:
+        raise ValueError("Must supply a connection")
 
     aggregate_reports = []
     forensic_reports = []
@@ -1071,22 +1061,13 @@ def get_dmarc_reports_from_inbox(connection=None,
         aggregate_reports = results["aggregate_reports"].copy()
         forensic_reports = results["forensic_reports"].copy()
 
-    if connection:
-        server = connection
-    else:
-        server = IMAPClient(host, user, password, port=port,
-                            ssl=ssl, verify=verify,
-                            timeout=timeout,
-                            max_retries=max_retries,
-                            initial_folder=reports_folder)
+    if not test and create_folders:
+        connection.create_folder(archive_folder)
+        connection.create_folder(aggregate_reports_folder)
+        connection.create_folder(forensic_reports_folder)
+        connection.create_folder(invalid_reports_folder)
 
-    if not test:
-        server.create_folder(archive_folder)
-        server.create_folder(aggregate_reports_folder)
-        server.create_folder(forensic_reports_folder)
-        server.create_folder(invalid_reports_folder)
-
-    messages = server.search()
+    messages = connection.fetch_messages(reports_folder)
     total_messages = len(messages)
     logger.debug("Found {0} messages in {1}".format(len(messages),
                                                     reports_folder))
@@ -1103,15 +1084,16 @@ def get_dmarc_reports_from_inbox(connection=None,
         logger.debug("Processing message {0} of {1}: UID {2}".format(
             i+1, message_limit, msg_uid
         ))
-        msg_content = server.fetch_message(msg_uid, parse=False)
-        sa = strip_attachment_payloads
+        msg_content = connection.fetch_message(msg_uid)
         try:
+            sa = strip_attachment_payloads
             parsed_email = parse_report_email(msg_content,
                                               nameservers=nameservers,
                                               dns_timeout=dns_timeout,
+                                              ip_db_path=ip_db_path,
                                               offline=offline,
                                               strip_attachment_payloads=sa,
-                                              server=server)
+                                              keep_alive=connection.keepalive)
             if parsed_email["report_type"] == "aggregate":
                 aggregate_reports.append(parsed_email["report"])
                 aggregate_report_msg_uids.append(msg_uid)
@@ -1124,12 +1106,12 @@ def get_dmarc_reports_from_inbox(connection=None,
                 if delete:
                     logger.debug(
                         "Deleting message UID {0}".format(msg_uid))
-                    server.delete_messages([msg_uid])
+                    connection.delete_message(msg_uid)
                 else:
                     logger.debug(
                         "Moving message UID {0} to {1}".format(
                             msg_uid, invalid_reports_folder))
-                    server.move_messages([msg_uid], invalid_reports_folder)
+                    connection.move_message(msg_uid, invalid_reports_folder)
 
     if not test:
         if delete:
@@ -1143,12 +1125,12 @@ def get_dmarc_reports_from_inbox(connection=None,
                     "Deleting message {0} of {1}: UID {2}".format(
                         i + 1, number_of_processed_msgs, msg_uid))
                 try:
-                    server.delete_messages([msg_uid])
+                    connection.delete_message(msg_uid)
 
                 except Exception as e:
                     message = "Error deleting message UID"
                     e = "{0} {1}: " "{2}".format(message, msg_uid, e)
-                    logger.error("IMAP error: {0}".format(e))
+                    logger.error("Mailbox error: {0}".format(e))
         else:
             if len(aggregate_report_msg_uids) > 0:
                 log_message = "Moving aggregate report messages from"
@@ -1163,12 +1145,12 @@ def get_dmarc_reports_from_inbox(connection=None,
                         "Moving message {0} of {1}: UID {2}".format(
                             i+1, number_of_agg_report_msgs, msg_uid))
                     try:
-                        server.move_messages([msg_uid],
-                                             aggregate_reports_folder)
+                        connection.move_message(msg_uid,
+                                                aggregate_reports_folder)
                     except Exception as e:
                         message = "Error moving message UID"
                         e = "{0} {1}: {2}".format(message, msg_uid, e)
-                        logger.error("IMAP error: {0}".format(e))
+                        logger.error("Mailbox error: {0}".format(e))
             if len(forensic_report_msg_uids) > 0:
                 message = "Moving forensic report messages from"
                 logger.debug(
@@ -1183,21 +1165,21 @@ def get_dmarc_reports_from_inbox(connection=None,
                         message,
                         i + 1, number_of_forensic_msgs, msg_uid))
                     try:
-                        server.move_messages([msg_uid],
-                                             forensic_reports_folder)
+                        connection.move_message(msg_uid,
+                                                forensic_reports_folder)
                     except Exception as e:
                         e = "Error moving message UID {0}: {1}".format(
                             msg_uid, e)
-                        logger.error("IMAP error: {0}".format(e))
+                        logger.error("Mailbox error: {0}".format(e))
     results = OrderedDict([("aggregate_reports", aggregate_reports),
                            ("forensic_reports", forensic_reports)])
 
-    total_messages = len(server.search())
+    total_messages = len(connection.fetch_messages(reports_folder))
 
     if not test and not batch_size and total_messages > 0:
         # Process emails that came in during the last run
-        results = get_dmarc_reports_from_inbox(
-            connection=server,
+        results = get_dmarc_reports_from_mailbox(
+            connection=connection,
             reports_folder=reports_folder,
             archive_folder=archive_folder,
             delete=delete,
@@ -1206,34 +1188,34 @@ def get_dmarc_reports_from_inbox(connection=None,
             dns_timeout=dns_timeout,
             strip_attachment_payloads=strip_attachment_payloads,
             results=results,
+            ip_db_path=ip_db_path,
             offline=offline
         )
 
     return results
 
 
-def watch_inbox(host, username, password, callback, port=None, ssl=True,
-                verify=True, reports_folder="INBOX",
+def watch_inbox(mailbox_connection: MailboxConnection,
+                callback: Callable,
+                reports_folder="INBOX",
                 archive_folder="Archive", delete=False, test=False,
-                idle_timeout=30, offline=False, nameservers=None,
+                check_timeout=30, ip_db_path=None,
+                offline=False, nameservers=None,
                 dns_timeout=6.0, strip_attachment_payloads=False,
                 batch_size=None):
     """
-    Use an IDLE IMAP connection to parse incoming emails, and pass the results
-    to a callback function
+    Watches the mailbox for new messages and
+      sends the results to a callback function
     Args:
-        host: The mail server hostname or IP address
-        username: The mail server username
-        password: The mail server password
+        mailbox_connection: The mailbox connection object
         callback: The callback function to receive the parsing results
-        port: The mail server port
-        ssl (bool): Use SSL/TLS
-        verify (bool): Verify the TLS/SSL certificate
         reports_folder: The IMAP folder where reports can be found
         archive_folder: The folder to move processed mail to
         delete (bool): Delete  messages after processing them
         test (bool): Do not move or delete messages after processing them
-        idle_timeout (int): Number of seconds to wait for a IMAP IDLE response
+        check_timeout (int): Number of seconds to wait for a IMAP IDLE response
+          or the number of seconds until the next mail check
+        ip_db_path (str): Path to a MMDB file from MaxMind or DBIP
         offline (bool): Do not query online for geolocation or DNS
         nameservers (list): A list of one or more nameservers to use
         (Cloudflare's public DNS resolvers by default)
@@ -1242,35 +1224,25 @@ def watch_inbox(host, username, password, callback, port=None, ssl=True,
         forensic report samples with None
         batch_size (int): Number of messages to read and process before saving
     """
-    sa = strip_attachment_payloads
 
-    def idle_callback(connection):
-        res = get_dmarc_reports_from_inbox(connection=connection,
-                                           reports_folder=reports_folder,
-                                           archive_folder=archive_folder,
-                                           delete=delete,
-                                           test=test,
-                                           offline=offline,
-                                           nameservers=nameservers,
-                                           dns_timeout=dns_timeout,
-                                           strip_attachment_payloads=sa,
-                                           batch_size=batch_size)
+    def check_callback(connection):
+        sa = strip_attachment_payloads
+        res = get_dmarc_reports_from_mailbox(connection=connection,
+                                             reports_folder=reports_folder,
+                                             archive_folder=archive_folder,
+                                             delete=delete,
+                                             test=test,
+                                             ip_db_path=ip_db_path,
+                                             offline=offline,
+                                             nameservers=nameservers,
+                                             dns_timeout=dns_timeout,
+                                             strip_attachment_payloads=sa,
+                                             batch_size=batch_size,
+                                             create_folders=False)
         callback(res)
 
-    while True:
-        try:
-            IMAPClient(host=host, username=username, password=password,
-                       port=port, ssl=ssl, verify=verify,
-                       initial_folder=reports_folder,
-                       idle_callback=idle_callback,
-                       idle_timeout=idle_timeout)
-        except (timeout, IMAPClientError):
-            logger.warning("IMAP connection timeout. Reconnecting...")
-            sleep(5)
-        except Exception as e:
-            logger.warning("IMAP connection error. {0}. "
-                           "Reconnecting...".format(e))
-            sleep(5)
+    mailbox_connection.watch(check_callback=check_callback,
+                             check_timeout=check_timeout)
 
 
 def save_output(results, output_directory="output",
@@ -1283,7 +1255,7 @@ def save_output(results, output_directory="output",
 
     Args:
         results (OrderedDict): Parsing results
-        output_directory (str): The patch to the directory to save in
+        output_directory (str): The path to the directory to save in
         aggregate_json_filename (str): Filename for the aggregate JSON file
         forensic_json_filename (str): Filename for the forensic JSON file
         aggregate_csv_filename (str): Filename for the aggregate CSV file
@@ -1420,7 +1392,7 @@ def email_results(results, host, mail_from, mail_to,
         attachment_filename (str): Override the default attachment filename
         message (str: Override the default plain text body
     """
-    logging.debug("Emailing report to: {0}".format(",".join(mail_to)))
+    logger.debug("Emailing report to: {0}".format(",".join(mail_to)))
     date_string = datetime.now().strftime("%Y-%m-%d")
     if attachment_filename:
         if not attachment_filename.lower().endswith(".zip"):
