@@ -13,13 +13,17 @@ import json
 from ssl import CERT_NONE, create_default_context
 from multiprocessing import Pool, Value
 from itertools import repeat
+import sys
 import time
 from tqdm import tqdm
 
-from parsedmarc import get_dmarc_reports_from_inbox, watch_inbox, \
+from parsedmarc import get_dmarc_reports_from_mailbox, watch_inbox, \
     parse_report_file, get_dmarc_reports_from_mbox, elastic, kafkaclient, \
     splunk, save_output, email_results, ParserError, __version__, \
-    InvalidDMARCReport, s3
+    InvalidDMARCReport, s3, syslog
+
+from parsedmarc.mail import IMAPConnection, MSGraphConnection, GmailConnection
+
 from parsedmarc.utils import is_mbox
 
 logger = logging.getLogger("parsedmarc")
@@ -31,11 +35,12 @@ def _str_to_list(s):
     return list(map(lambda i: i.lstrip(), _list))
 
 
-def cli_parse(file_path, sa, nameservers, dns_timeout, offline,
-              parallel=False):
+def cli_parse(file_path, sa, nameservers, dns_timeout,
+              ip_db_path, offline, parallel=False):
     """Separated this function for multiprocessing"""
     try:
         file_results = parse_report_file(file_path,
+                                         ip_db_path=ip_db_path,
                                          offline=offline,
                                          nameservers=nameservers,
                                          dns_timeout=dns_timeout,
@@ -61,6 +66,7 @@ def _main():
         output_str = "{0}\n".format(json.dumps(reports_,
                                                ensure_ascii=False,
                                                indent=2))
+
         if not opts.silent:
             print(output_str)
         if opts.kafka_hosts:
@@ -87,6 +93,14 @@ def _main():
                 )
             except Exception as error_:
                 logger.error("S3 Error: {0}".format(error_.__str__()))
+        if opts.syslog_server:
+            try:
+                syslog_client = syslog.SyslogClient(
+                    server_name=opts.syslog_server,
+                    server_port=int(opts.syslog_port),
+                )
+            except Exception as error_:
+                logger.error("Syslog Error: {0}".format(error_.__str__()))
         if opts.save_aggregate:
             for report in reports_["aggregate_reports"]:
                 try:
@@ -117,6 +131,11 @@ def _main():
                         s3_client.save_aggregate_report_to_s3(report)
                 except Exception as error_:
                     logger.error("S3 Error: {0}".format(error_.__str__()))
+                try:
+                    if opts.syslog_server:
+                        syslog_client.save_aggregate_report_to_syslog(report)
+                except Exception as error_:
+                    logger.error("Syslog Error: {0}".format(error_.__str__()))
             if opts.hec:
                 try:
                     aggregate_reports_ = reports_["aggregate_reports"]
@@ -156,6 +175,11 @@ def _main():
                         s3_client.save_forensic_report_to_s3(report)
                 except Exception as error_:
                     logger.error("S3 Error: {0}".format(error_.__str__()))
+                try:
+                    if opts.syslog_server:
+                        syslog_client.save_forensic_report_to_syslog(report)
+                except Exception as error_:
+                    logger.error("Syslog Error: {0}".format(error_.__str__()))
             if opts.hec:
                 try:
                     forensic_reports_ = reports_["forensic_reports"]
@@ -216,6 +240,8 @@ def _main():
 
     args = arg_parser.parse_args()
 
+    default_gmail_api_scope = 'https://www.googleapis.com/auth/gmail.modify'
+
     opts = Namespace(file_path=args.file_path,
                      config_file=args.config_file,
                      offline=args.offline,
@@ -232,6 +258,12 @@ def _main():
                      verbose=args.verbose,
                      save_aggregate=False,
                      save_forensic=False,
+                     mailbox_reports_folder="INBOX",
+                     mailbox_archive_folder="Archive",
+                     mailbox_watch=False,
+                     mailbox_delete=False,
+                     mailbox_test=False,
+                     mailbox_batch_size=None,
                      imap_host=None,
                      imap_skip_certificate_verification=False,
                      imap_ssl=True,
@@ -240,12 +272,11 @@ def _main():
                      imap_max_retries=4,
                      imap_user=None,
                      imap_password=None,
-                     imap_reports_folder="INBOX",
-                     imap_archive_folder="Archive",
-                     imap_watch=False,
-                     imap_delete=False,
-                     imap_test=False,
-                     imap_batch_size=None,
+                     graph_user=None,
+                     graph_password=None,
+                     graph_client_id=None,
+                     graph_client_secret=None,
+                     graph_mailbox=None,
                      hec=None,
                      hec_token=None,
                      hec_index=None,
@@ -253,7 +284,7 @@ def _main():
                      elasticsearch_hosts=None,
                      elasticsearch_timeout=60,
                      elasticsearch_number_of_shards=1,
-                     elasticsearch_number_of_replicas=1,
+                     elasticsearch_number_of_replicas=0,
                      elasticsearch_index_suffix=None,
                      elasticsearch_ssl=True,
                      elasticsearch_ssl_cert_path=None,
@@ -279,9 +310,16 @@ def _main():
                      smtp_message="Please see the attached DMARC results.",
                      s3_bucket=None,
                      s3_path=None,
+                     syslog_server=None,
+                     syslog_port=None,
+                     gmail_api_credentials_file=None,
+                     gmail_api_token_file=None,
+                     gmail_api_include_spam_trash=False,
+                     gmail_api_scopes=[],
                      log_file=args.log_file,
                      n_procs=1,
-                     chunk_size=1
+                     chunk_size=1,
+                     ip_db_path=None
                      )
     args = arg_parser.parse_args()
 
@@ -296,10 +334,10 @@ def _main():
         if "general" in config.sections():
             general_config = config["general"]
             if "offline" in general_config:
-                opts.offline = general_config["offline"]
+                opts.offline = general_config.getboolean("offline")
             if "strip_attachment_payloads" in general_config:
-                opts.strip_attachment_payloads = general_config[
-                    "strip_attachment_payloads"]
+                opts.strip_attachment_payloads = general_config.getboolean(
+                    "strip_attachment_payloads")
             if "output" in general_config:
                 opts.output = general_config["output"]
             if "aggregate_json_filename" in general_config:
@@ -334,6 +372,28 @@ def _main():
                 opts.n_procs = general_config.getint("n_procs")
             if "chunk_size" in general_config:
                 opts.chunk_size = general_config.getint("chunk_size")
+            if "ip_db_path" in general_config:
+                opts.ip_db_path = general_config["ip_db_path"]
+            else:
+                opts.ip_db_path = None
+
+        if "mailbox" in config.sections():
+            mailbox_config = config["mailbox"]
+            if "reports_folder" in mailbox_config:
+                opts.mailbox_reports_folder = mailbox_config["reports_folder"]
+            if "archive_folder" in mailbox_config:
+                opts.mailbox_archive_folder = mailbox_config["archive_folder"]
+            if "watch" in mailbox_config:
+                opts.mailbox_watch = mailbox_config.getboolean("watch")
+            if "delete" in mailbox_config:
+                opts.mailbox_delete = mailbox_config.getboolean("delete")
+            if "test" in mailbox_config:
+                opts.mailbox_test = mailbox_config.getboolean("test")
+            if "batch_size" in mailbox_config:
+                opts.mailbox_batch_size = mailbox_config.getint("batch_size")
+            else:
+                opts.mailbox_batch_size = None
+
         if "imap" in config.sections():
             imap_config = config["imap"]
             if "host" in imap_config:
@@ -367,20 +427,37 @@ def _main():
                                 "imap config section")
                 exit(-1)
 
-            if "reports_folder" in imap_config:
-                opts.imap_reports_folder = imap_config["reports_folder"]
-            if "archive_folder" in imap_config:
-                opts.imap_archive_folder = imap_config["archive_folder"]
-            if "watch" in imap_config:
-                opts.imap_watch = imap_config.getboolean("watch")
-            if "delete" in imap_config:
-                opts.imap_delete = imap_config.getboolean("delete")
-            if "test" in imap_config:
-                opts.imap_test = imap_config.getboolean("test")
-            if "batch_size" in imap_config:
-                opts.imap_batch_size = imap_config.getint("batch_size")
+        if "msgraph" in config.sections():
+            graph_config = config["msgraph"]
+            if "user" in graph_config:
+                opts.graph_user = graph_config["user"]
             else:
-                opts.imap_batch_size = None
+                logger.critical("user setting missing from the "
+                                "msgraph config section")
+                exit(-1)
+            if "password" in graph_config:
+                opts.graph_password = graph_config["password"]
+            else:
+                logger.critical("password setting missing from the "
+                                "msgraph config section")
+                exit(-1)
+
+            if "client_id" in graph_config:
+                opts.graph_client_id = graph_config["client_id"]
+            else:
+                logger.critical("client_id setting missing from the "
+                                "msgraph config section")
+                exit(-1)
+
+            if "client_secret" in graph_config:
+                opts.graph_client_secret = graph_config["client_secret"]
+            else:
+                logger.critical("client_secret setting missing from the "
+                                "msgraph config section")
+                exit(-1)
+            if "mailbox" in graph_config:
+                opts.graph_mailbox = graph_config["mailbox"]
+
         if "elasticsearch" in config:
             elasticsearch_config = config["elasticsearch"]
             if "hosts" in elasticsearch_config:
@@ -463,7 +540,7 @@ def _main():
                                 "kafka config section")
                 exit(-1)
             if "ssl" in kafka_config:
-                opts.kafka_ssl = kafka_config["ssl"].getboolean()
+                opts.kafka_ssl = kafka_config.getboolean("ssl")
             if "skip_certificate_verification" in kafka_config:
                 kafka_verify = kafka_config.getboolean(
                     "skip_certificate_verification")
@@ -488,7 +565,7 @@ def _main():
                                 "smtp config section")
                 exit(-1)
             if "port" in smtp_config:
-                opts.smtp_port = smtp_config["port"]
+                opts.smtp_port = smtp_config.getint("port")
             if "ssl" in smtp_config:
                 opts.smtp_ssl = smtp_config.getboolean("ssl")
             if "skip_certificate_verification" in smtp_config:
@@ -539,15 +616,36 @@ def _main():
                     opts.s3_path = opts.s3_path[:-1]
             else:
                 opts.s3_path = ""
+        if "syslog" in config.sections():
+            syslog_config = config["syslog"]
+            if "server" in syslog_config:
+                opts.syslog_server = syslog_config["server"]
+            else:
+                logger.critical("server setting missing from the "
+                                "syslog config section")
+                exit(-1)
+            if "port" in syslog_config:
+                opts.syslog_port = syslog_config["port"]
+            else:
+                opts.syslog_port = 514
+
+        if "gmail_api" in config.sections():
+            gmail_api_config = config["gmail_api"]
+            opts.gmail_api_credentials_file = \
+                gmail_api_config.get("credentials_file")
+            opts.gmail_api_token_file = \
+                gmail_api_config.get("token_file", ".token")
+            opts.gmail_api_include_spam_trash = \
+                gmail_api_config.getboolean("include_spam_trash", False)
+            opts.gmail_api_scopes = \
+                gmail_api_config.get("scopes",
+                                     default_gmail_api_scope).split(',')
 
-    logging.basicConfig(level=logging.WARNING)
     logger.setLevel(logging.WARNING)
 
     if opts.verbose:
-        logging.basicConfig(level=logging.INFO)
         logger.setLevel(logging.INFO)
     if opts.debug:
-        logging.basicConfig(level=logging.DEBUG)
         logger.setLevel(logging.DEBUG)
     if opts.log_file:
         fh = logging.FileHandler(opts.log_file)
@@ -556,11 +654,15 @@ def _main():
             '%(levelname)s - [%(filename)s:%(lineno)d] - %(message)s')
         fh.setFormatter(formatter)
         logger.addHandler(fh)
-    if opts.imap_host is None and len(opts.file_path) == 0:
-        logger.error("You must supply input files, or an IMAP configuration")
+
+    if opts.imap_host is None \
+            and opts.graph_user is None \
+            and opts.gmail_api_credentials_file is None \
+            and len(opts.file_path) == 0:
+        logger.error("You must supply input files or a mailbox connection")
         exit(1)
 
-    logger.info("Starting dmarcparse")
+    logger.info("Starting parsedmarc")
 
     if opts.save_aggregate or opts.save_forensic:
         try:
@@ -623,14 +725,19 @@ def _main():
                                      repeat(opts.strip_attachment_payloads),
                                      repeat(opts.nameservers),
                                      repeat(opts.dns_timeout),
+                                     repeat(opts.ip_db_path),
                                      repeat(opts.offline),
                                      repeat(opts.n_procs >= 1)),
                                  opts.chunk_size)
-    pbar = tqdm(total=len(file_paths))
-    while not results.ready():
-        pbar.update(counter.value - pbar.n)
-        time.sleep(0.1)
-    pbar.close()
+    if sys.stdout.isatty():
+        pbar = tqdm(total=len(file_paths))
+        while not results.ready():
+            pbar.update(counter.value - pbar.n)
+            time.sleep(0.1)
+        pbar.close()
+    else:
+        while not results.ready():
+            time.sleep(0.1)
     results = results.get()
     pool.close()
     pool.join()
@@ -646,23 +753,24 @@ def _main():
                 forensic_reports.append(result[0]["report"])
 
     for mbox_path in mbox_paths:
-        reports = get_dmarc_reports_from_mbox(mbox_path, opts.nameservers,
-                                              opts.dns_timeout,
-                                              opts.strip_attachment_payloads,
-                                              opts.offline, False)
+        strip = opts.strip_attachment_payloads
+        reports = get_dmarc_reports_from_mbox(mbox_path,
+                                              nameservers=opts.nameservers,
+                                              dns_timeout=opts.dns_timeout,
+                                              strip_attachment_payloads=strip,
+                                              ip_db_path=opts.ip_db_path,
+                                              offline=opts.offline,
+                                              parallel=False)
         aggregate_reports += reports["aggregate_reports"]
         forensic_reports += reports["forensic_reports"]
 
+    mailbox_connection = None
     if opts.imap_host:
         try:
             if opts.imap_user is None or opts.imap_password is None:
                 logger.error("IMAP user and password must be specified if"
                              "host is specified")
 
-            rf = opts.imap_reports_folder
-            af = opts.imap_archive_folder
-            ns = opts.nameservers
-            sa = opts.strip_attachment_payloads
             ssl = True
             verify = True
             if opts.imap_skip_certificate_verification:
@@ -670,7 +778,8 @@ def _main():
                 verify = False
             if opts.imap_ssl is False:
                 ssl = False
-            reports = get_dmarc_reports_from_inbox(
+
+            mailbox_connection = IMAPConnection(
                 host=opts.imap_host,
                 port=opts.imap_port,
                 ssl=ssl,
@@ -679,21 +788,69 @@ def _main():
                 max_retries=opts.imap_max_retries,
                 user=opts.imap_user,
                 password=opts.imap_password,
-                reports_folder=rf,
-                archive_folder=af,
-                delete=opts.imap_delete,
+            )
+
+        except Exception as error:
+            logger.error("IMAP Error: {0}".format(error.__str__()))
+            exit(1)
+
+    if opts.graph_user:
+        try:
+            mailbox = opts.graph_mailbox or opts.graph_user
+            mailbox_connection = MSGraphConnection(
+                client_id=opts.graph_client_id,
+                client_secret=opts.graph_client_secret,
+                username=opts.graph_user,
+                password=opts.graph_password,
+                mailbox=mailbox
+            )
+
+        except Exception as error:
+            logger.error("MS Graph Error: {0}".format(error.__str__()))
+            exit(1)
+
+    if opts.gmail_api_credentials_file:
+        if opts.mailbox_delete:
+            if 'https://mail.google.com/' not in opts.gmail_api_scopes:
+                logger.error("Message deletion requires scope"
+                             " 'https://mail.google.com/'. "
+                             "Add the scope and remove token file "
+                             "to acquire proper access.")
+                opts.mailbox_delete = False
+
+        try:
+            mailbox_connection = GmailConnection(
+                credentials_file=opts.gmail_api_credentials_file,
+                token_file=opts.gmail_api_token_file,
+                scopes=opts.gmail_api_scopes,
+                include_spam_trash=opts.gmail_api_include_spam_trash,
+                reports_folder=opts.mailbox_reports_folder
+            )
+
+        except Exception as error:
+            logger.error("Gmail API Error: {0}".format(error.__str__()))
+            exit(1)
+
+    if mailbox_connection:
+        try:
+            reports = get_dmarc_reports_from_mailbox(
+                connection=mailbox_connection,
+                delete=opts.mailbox_delete,
+                batch_size=opts.mailbox_batch_size,
+                reports_folder=opts.mailbox_reports_folder,
+                archive_folder=opts.mailbox_archive_folder,
+                ip_db_path=opts.ip_db_path,
                 offline=opts.offline,
-                nameservers=ns,
-                test=opts.imap_test,
-                strip_attachment_payloads=sa,
-                batch_size=opts.imap_batch_size
+                nameservers=opts.nameservers,
+                test=opts.mailbox_test,
+                strip_attachment_payloads=opts.strip_attachment_payloads,
             )
 
             aggregate_reports += reports["aggregate_reports"]
             forensic_reports += reports["forensic_reports"]
 
         except Exception as error:
-            logger.error("IMAP Error: {0}".format(error.__str__()))
+            logger.error("Mailbox Error: {0}".format(error.__str__()))
             exit(1)
 
     results = OrderedDict([("aggregate_reports", aggregate_reports),
@@ -722,33 +879,22 @@ def _main():
             logger.error("{0}".format(error.__str__()))
             exit(1)
 
-    if opts.imap_host and opts.imap_watch:
+    if mailbox_connection and opts.mailbox_watch:
         logger.info("Watching for email - Quit with ctrl-c")
-        ssl = True
-        verify = True
-        if opts.imap_skip_certificate_verification:
-            logger.debug("Skipping IMAP certificate verification")
-            verify = False
-        if opts.imap_ssl is False:
-            ssl = False
+
         try:
-            sa = opts.strip_attachment_payloads
             watch_inbox(
-                opts.imap_host,
-                opts.imap_user,
-                opts.imap_password,
-                process_reports,
-                port=opts.imap_port,
-                ssl=ssl,
-                verify=verify,
-                reports_folder=opts.imap_reports_folder,
-                archive_folder=opts.imap_archive_folder,
-                delete=opts.imap_delete,
-                test=opts.imap_test,
+                mailbox_connection=mailbox_connection,
+                callback=process_reports,
+                reports_folder=opts.mailbox_reports_folder,
+                archive_folder=opts.mailbox_archive_folder,
+                delete=opts.mailbox_delete,
+                test=opts.mailbox_test,
                 nameservers=opts.nameservers,
                 dns_timeout=opts.dns_timeout,
-                strip_attachment_payloads=sa,
-                batch_size=opts.imap_batch_size,
+                strip_attachment_payloads=opts.strip_attachment_payloads,
+                batch_size=opts.mailbox_batch_size,
+                ip_db_path=opts.ip_db_path,
                 offline=opts.offline)
         except FileExistsError as error:
             logger.error("{0}".format(error.__str__()))
